﻿/*:
 * @target MZ
 * @plugindesc カスタムウィンドウを表示・閉じるプラグイン
 * @author Becky
 * @url https://games-p.com/
 * @help カスタムウィンドウを表示したい、または閉じたいイベント内でプラグインコマンド「CustomWindow」を実行して下さい。
 * @command カスタムウィンドウを開く
 * @desc カスタムウィンドウを表示します。
 * @arg x
 * @type number
 * @desc ウィンドウのX座標
 * @default 0
 * @arg y
 * @type number
 * @desc ウィンドウのY座標
 * @default 0
 *
 * @arg width
 * @type number
 * @min 60
 * @text 幅
 * @desc ウィンドウの幅
 * @default 400
 *
 * @arg height
 * @type number
 * @min 60
 * @text 高さ
 * @desc ウィンドウの高さ
 * @default 400
 *
 * @arg variableId
 * @text 表示する変数ID
 * @type variable
 * @desc ウィンドウに表示する変数のIDを指定します。
 * @default 1
 *
 * @command カスタムウィンドウを閉じる
 * @desc カスタムウィンドウを閉じます。
 */

(() => {
    let customWindow = null;

    // カスタムウィンドウクラス
    class CustomWindow extends Window_Base {
        constructor(rect, variableId) {
            super(rect);
            this._variableId = variableId;
            this.refresh();
        }

        refresh() {
            this.contents.clear();
            const value = $gameVariables.value(this._variableId);
            this.drawTextEx(`${value}`, 0, 0, this.width - this.padding * 2);
        }
    }
    
    // カスタムウィンドウを開くプラグインコマンドの登録
    PluginManager.registerCommand('CustomWindow', 'カスタムウィンドウを開く', args => {
        const scene = SceneManager._scene;
        if (scene && scene._windowLayer) {
            const x = Number(args.x) || 0;
            const y = Number(args.y) || 0;
            const width = Number(args.width) || 400;
            const height = Number(args.height) || 400;
            const rect = new Rectangle(x, y, width, height);
            const variableId = Number(args.variableId) || 1;
            
            if (customWindow) {
                scene._windowLayer.removeChild(customWindow);
                customWindow.destroy();
            }

            customWindow = new CustomWindow(rect, variableId);
            scene._windowLayer.addChild(customWindow);
        }
    });

    // カスタムウィンドウを閉じるプラグインコマンドの登録
    PluginManager.registerCommand('CustomWindow', 'カスタムウィンドウを閉じる', args => {
        const scene = SceneManager._scene;
        if (scene && scene._windowLayer && customWindow) {
            scene._windowLayer.removeChild(customWindow);
            customWindow.destroy();
            customWindow = null;
        }
    });
})();