//=============================================================================
// FC_AjustFontMZ .js
//=============================================================================

/*:
 * @target MZ
 * @plugindesc Adjust around the font.
 * @author Koji Marumugi
 * 
 * @param NoOutline
 * @text Erase the outline.
 * @desc Whether or not to remove the font outlines. (1: remove, 0: leave as is)
 * @default 0
 * 
 * @param SystemIsNomal
 * @text Set the system color to normal.
 * @desc Sets the system color of the font to normal color. (1:ON, 0:OFF)
 * @default 0
 * @type number
 * 
 * @param CrisisIsNomal
 * @text Set the crisis color to normal.
 * @desc Sets the crisis color of the font to normal color. (1:ON, 0:OFF)
 * @default 0
 * @type number
 * 
 * @param DeathIsNomal
 * @text Set the death color to normal.
 * @desc Sets the death color of the font to normal color. (1:ON, 0:OFF)
 * @default 0
 * @type number
 * 
 * @param GaugeBackIsBlack
 * @text Set the background color of each gauge to black.
 * @desc Sets the background color of each gauge to black. (1:ON, 0:OFF)
 * @default 0
 * @type number
 * 
 * @param HpGaugeIsNomal
 * @text Set the color of the HP gauge to normal.
 * @desc Sets the color of the HP gauge to the normal color. (1:ON, 0:OFF)
 * @default 0
 * @type number
 * 
 * @param MpGaugeIsNomal
 * @text Set the color of the MP gauge to normal.
 * @desc Sets the color of the MP gauge to the normal color. (1:ON, 0:OFF)
 * @default 0
 * @type number
 * 
 * @param MpCostIsNomal
 * @text Set the MP cost color to normal.
 * @desc Sets the MP cost color of the font to normal color. (1:ON, 0:OFF)
 * @default 0
 * @type number
 * 
 * @param PowerUpIsNomal
 * @text Set the power up color to normal.
 * @desc Sets the power up color of the font to normal color. (1:ON, 0:OFF)
 * @default 0
 * @type number
 * 
 * @param PowerDownIsNomal
 * @text Set the power down color to normal.
 * @desc Sets the power down color of the font to normal color. (1:ON, 0:OFF)
 * @default 0
 * @type number
 * 
 * @param TpGaugeIsNomal
 * @text Set the color of the TP gauge to normal.
 * @desc Sets the color of the TP gauge to the normal color. (1:ON, 0:OFF)
 * @default 0
 * @type number
 * 
 * @param TpCostIsNomal
 * @text Set the TP cost color to normal.
 * @desc Sets the TP cost color of the font to normal color. (1:ON, 0:OFF)
 * @default 0
 * @type number
 * 
 * @param CtGaugeIsNomal
 * @text @text Set the color of the CT gauge to normal.
 * @desc Sets the color of the CT gauge to the normal color. (1:ON, 0:OFF)
 * @default 0
 * @type number
 * 
 * @param NoBackground
 * @textErase the selectable range.
 * @desc Removes the translucent color that indicates the selectable range.（1:ON、0:OFF)
 * @default 0
 * @type number
 *
 * @help This plugin does not provide plugin commands.
 * 
 * [License]
 * this plugin is released under MIT license.
 * http://opensource.org/licenses/mit-license.php
 */

/*:ja
 * @target MZ
 * @plugindesc フォントまわりを調節します。
 * @author Koji Marumugi
 * 
 * @param NoOutline
 * @text アウトラインを消す
 * @desc フォントのアウトラインを消すかどうか。（1:消す、0:そのまま)
 * @default 0
 * @type number
 * 
 * @param SystemIsNomal
 * @text システムカラーをノーマルにする
 * @desc フォントのシステムカラーをノーマルカラーにします。（1:ON、0:OFF)
 * @default 0
 * @type number
 * 
 * @param CrisisIsNomal
 * @text 瀕死時の色をノーマルにする
 * @desc フォントの瀕死時のカラーをノーマルカラーにします。（1:ON、0:OFF)
 * @default 0
 * @type number
 * 
 * @param DeathIsNomal
 * @text 死亡時の色をノーマルにする
 * @desc フォントの死亡時のカラーをノーマルカラーにします。（1:ON、0:OFF)
 * @default 0
 * @type number
 * 
 * @param GaugeBackIsBlack
 * @text ゲージの背景色を黒にする
 * @desc フォントの各ゲージの背景色を黒にします。（1:ON、0:OFF)
 * @default 0
 * @type number
 * 
 * @param HpGaugeIsNomal
 * @text HPゲージの色をノーマルにする
 * @desc フォントのHPゲージの色をノーマルカラーにします。（1:ON、0:OFF)
 * @default 0
 * @type number
 * 
 * @param MpGaugeIsNomal
 * @text MPゲージの色をノーマルにする
 * @desc フォントのMPゲージの色をノーマルカラーにします。（1:ON、0:OFF)
 * @default 0
 * @type number
 * 
 * @param MpCostIsNomal
 * @text MPのコストの色をノーマルにする
 * @desc フォントのMPのコストの色をノーマルカラーにします。（1:ON、0:OFF)
 * @default 0
 * @type number
 * 
 * @param PowerUpIsNomal
 * @text 上昇値の色をノーマルにする
 * @desc フォントの上昇値の色をノーマルカラーにします。（1:ON、0:OFF)
 * @default 0
 * @type number
 * 
 * @param PowerDownIsNomal
 * @text 下降値の色をノーマルにする
 * @desc フォントの下降値の色をノーマルカラーにします。（1:ON、0:OFF)
 * @default 0
 * @type number
 * 
 * @param TpGaugeIsNomal
 * @text TPゲージの色をノーマルにする
 * @desc フォントのTPゲージの色をノーマルカラーにします。（1:ON、0:OFF)
 * @default 0
 * @type number
 * 
 * @param TpCostIsNomal
 * @text TPのコストの色をノーマルにする
 * @desc フォントのTPのコストの色をノーマルカラーにします。（1:ON、0:OFF)
 * @default 0
 * @type number
 * 
 * @param CtGaugeIsNomal
 * @text CTゲージの色をノーマルにする
 * @desc フォントのCTゲージの色をノーマルカラーにします。（1:ON、0:OFF)
 * @default 0
 * @type number
 * 
 * @param NoBackground
 * @text 選択可能範囲を消す
 * @desc 選択可能範囲を示す半透明の色を消去します。（1:ON、0:OFF)
 * @default 0
 * @type number
 *
 * @help このプラグインには、プラグインコマンドはありません。
 * 
 * ■ライセンス表記
 * このプラグインは MIT ライセンスで配布されます。
 * ご自由にお使いください。
 * http://opensource.org/licenses/mit-license.php
 */

(()=>{
    'use strict';

    const name = 'FC_AjustFontMZ';
    const parameters = PluginManager.parameters(name);
    let noOutline = Number(parameters['NoOutline'] || 0);
    let system = Number(parameters['SystemIsNomal'] || 0);
    let crisis = Number(parameters['CrisisIsNomal'] || 0);
    let death = Number(parameters['DeathIsNomal'] || 0);
    let gaugeBack = Number(parameters['GaugeBackIsBlack'] || 0);
    let hpGauge = Number(parameters['HpGaugeIsNomal'] || 0);
    let mpGauge = Number(parameters['MpGaugeIsNomal'] || 0);
    let mpCost = Number(parameters['MpCostIsNomal'] || 0);
    let powerUp = Number(parameters['PowerUpIsNomal'] || 0);
    let powerDown = Number(parameters['PowerDownIsNomal'] || 0);
    let tpGauge = Number(parameters['TpGaugeIsNomal'] || 0);
    let tpCost = Number(parameters['TpCostIsNomal'] || 0);
    let ctGauge = Number(parameters['CtGaugeIsNomal'] || 0);
    let noBack = Number(parameters['NoBackground'] || 0);
   
    if(noBack === 1) {
        Window_Selectable.prototype.drawBackgroundRect = function(rect) {
        }
    };

    if(system === 1) {
        const _colorManager_systemColor = ColorManager.systemColor;
        ColorManager.systemColor = function() {
            _colorManager_systemColor.apply(this, arguments);
            return this.textColor(0);
        }
    };

    if(crisis === 1) {
        const _colorManager_crisisColor = ColorManager.crisisColor;
        ColorManager.crisisColor = function() {
            _colorManager_crisisColor.apply(this, arguments);
            return this.textColor(0);
        }
    };

    if(death === 1) {
        const _colorManager_deathColor = ColorManager.deathColor;
        ColorManager.deathColor = function() {
            _colorManager_deathColor.apply(this, arguments);
            return this.textColor(0);
        }
    };

    if(gaugeBack === 1) {
        const _colorManager_gaugeBackColor = ColorManager.gaugeBackColor;
        ColorManager.gaugeBackColor = function() {
            _colorManager_gaugeBackColor.apply(this, arguments);
            return this.textColor(15);
        }
    };

    if(hpGauge === 1) {
        const _colorManager_hpGaugeColor1 = ColorManager.hpGaugeColor1;
        ColorManager.hpGaugeColor1 = function() {
            _colorManager_hpGaugeColor1.apply(this, arguments);
            return this.textColor(0);
        }

        const _colorManager_hpGaugeColor2 = ColorManager.hpGaugeColor2;
        ColorManager.hpGaugeColor2 = function() {
            _colorManager_hpGaugeColor2.apply(this, arguments);
            return this.textColor(0);
        }
    };

    if(mpGauge === 1) {
        const _colorManager_mpGaugeColor1 = ColorManager.mpGaugeColor1;
        ColorManager.mpGaugeColor1 = function() {
            _colorManager_mpGaugeColor1.apply(this, arguments);
            return this.textColor(0);
        }

        const _colorManager_mpGaugeColor2 = ColorManager.mpGaugeColor2;
        ColorManager.mpGaugeColor2 = function() {
            _colorManager_mpGaugeColor2.apply(this, arguments);
            return this.textColor(0);
        }
    };

    if(mpCost === 1) {
        const _colorManager_mpCostColor = ColorManager.mpCostColor;
        ColorManager.mpCostColor = function() {
            _colorManager_mpCostColor.apply(this, arguments);
            return this.textColor(0);
        }
    };

    if(powerUp === 1) {
        const _colorManager_powerUpColor = ColorManager.powerUpColor;
        ColorManager.powerUpColor = function() {
            _colorManager_powerUpColor.apply(this, arguments);
            return this.textColor(0);
        }
    };

    if(powerDown === 1) {
        const _colorManager_powerDownColor = ColorManager.powerDownColor;
        ColorManager.powerDownColor = function() {
            _colorManager_powerDownColor.apply(this, arguments);
            return this.textColor(0);
        }
    };

    if(tpGauge === 1) {
        const _colorManager_tpGaugeColor1 = ColorManager.tpGaugeColor1;
        ColorManager.tpGaugeColor1 = function() {
            _colorManager_tpGaugeColor1.apply(this, arguments);
            return this.textColor(0);
        }

        const _colorManager_tpGaugeColor2 = ColorManager.tpGaugeColor2;
        ColorManager.tpGaugeColor2 = function() {
            _colorManager_tpGaugeColor2.apply(this, arguments);
            return this.textColor(0);
        }
    };

    if(tpCost === 1) {
        const _colorManager_tpCostColor = ColorManager.tpCostColor;
        ColorManager.tpCostColor = function() {
            _colorManager_tpCostColor.apply(this, arguments);
            return this.textColor(0);
        }
    };

    if(ctGauge === 1) {
        const _colorManager_ctGaugeColor1 = ColorManager.ctGaugeColor1;
        ColorManager.ctGaugeColor1 = function() {
            _colorManager_ctGaugeColor1.apply(this, arguments);
            return this.textColor(0);
        }

        const _colorManager_ctGaugeColor2 = ColorManager.ctGaugeColor2;
        ColorManager.ctGaugeColor2 = function() {
            _colorManager_ctGaugeColor2.apply(this, arguments);
            return this.textColor(0);
        }
    };

    if(noOutline === 1) {
        Bitmap.prototype.initialize = function(width, height) {
            if(!this._defer){
                this._createCanvas(width, height);
            }
        
            this._image = null;
            this._url = '';
            this._paintOpacity = 255;
            this._smooth = false;
            this._loadListeners = [];
            this._loadingState = 'none';
            this._decodeAfterRequest = false;
        
            /**
             * Cache entry, for images. In all cases _url is the same as cacheEntry.key
             * @type CacheEntry
             */
            this.cacheEntry = null;
        
            /**
             * The face name of the font.
             *
             * @property fontFace
             * @type String
             */
            this.fontFace = 'GameFont';
        
            /**
             * The size of the font in pixels.
             *
             * @property fontSize
             * @type Number
             */
            this.fontSize = 28;
        
            /**
             * Whether the font is italic.
             *
             * @property fontItalic
             * @type Boolean
             */
            this.fontItalic = false;
        
            /**
             * The color of the text in CSS format.
             *
             * @property textColor
             * @type String
             */
            this.textColor = '#ffffff';
        
            /**
             * The color of the outline of the text in CSS format.
             *
             * @property outlineColor
             * @type String
             */
            this.outlineColor = 'rgba(0, 0, 0, 0)';
        
            /**
             * The width of the outline of the text.
             *
             * @property outlineWidth
             * @type Number
             */
            this.outlineWidth = 0;

        };
        // _bitmap_initialize.apply(this, arguments);
    };

})();