//=============================================================================
// FC_SVCharaPositionMZ.js
//=============================================================================

/*:
 * @target MZ
 * @plugindesc Adjust the position of the SV Actors.
 * @author Koji Marumugi
 * 
 * @param x
 * @text Basic X coordinate
 * @desc X coordinate of the topmost character. (Default 600)
 * @default 600
 * @type number
 * 
 * @param spaceX
 * @text Size to be shifted horizontally
 * @desc The number of pixels to shift horizontally from top to bottom. (Default 32)
 * @default 32
 * @type number
 * 
 * @param y
 * @text Basic Y coordinate
 * @desc Y-coordinate of the topmost character when the bottom edge of the battle background image is set to 0. (Default 120)
 * @default 120
 * @type number
 *
 * @param spaceY
 * @text Space
 * @desc Vertical spacing between characters. (Default -15)
 * @default -15
 * @type number
 *
 * @help This plugin does not provide plugin commands.
 * 
 * [License]
 * this plugin is released under MIT license.
 * http://opensource.org/licenses/mit-license.php
 */

/*:ja
 * @target MZ
 * @plugindesc SVキャラの位置を調整します
 * @author Koji Marumugi
 * 
 * @param x
 * @text 基本のＸ座標
 * @desc 一番上のキャラのＸ座標。（デフォルト600）
 * @default 600
 * @type number
 * 
 * @param spaceX
 * @text 横方向にずらすサイズ
 * @desc 上から下にかけて横方向に何ピクセルずらすか。（デフォルト32）
 * @default 32
 * @type number
 * 
 * @param y
 * @text 基本のＹ座標
 * @desc 戦闘背景画像の下端を0とした時の、一番上のキャラのＹ座標。（デフォルト120）
 * @default 120
 * @type number
 *
 * @param spaceY
 * @text 間隔
 * @desc キャラ同士の縦方向の間隔。（デフォルト-15）
 * @default -15
 * @type number
 *
 * @help このプラグインには、プラグインコマンドはありません。
 * 
 * ■ライセンス表記
 * このプラグインは MIT ライセンスで配布されます。
 * ご自由にお使いください。
 * http://opensource.org/licenses/mit-license.php
 */

(()=>{
    'use strict';

    const name = 'FC_SVCharaPositionMZ';
    const parameters = PluginManager.parameters(name);
    let x = Number(parameters['x'] || 0);
    let spaceX = Number(parameters['spaceX'] || 0);
    let y = Number(parameters['y'] || 0);
    let spaceY = Number(parameters['spaceY'] || 0);

    const _sprite_Actor_setActorHome = Sprite_Actor.prototype.setActorHome;
    Sprite_Actor.prototype.setActorHome = function(index) {
        const ret = _sprite_Actor_setActorHome.apply(this, arguments);

        this.setHome(x + index * (1 + spaceX), 180 + y + index * (63 + spaceY));

        return ret;

    }

})();