//=============================================================================
// FC_ShadowPenMZ.js
//=============================================================================

/*:
 * @target MZ
 * @plugindesc You can make the shadow pen completely transparent or black.
 * @author Koji Marumugi
 * 
 * @param choice
 * @text Transparent or black?
 * @desc Select whether you want the shadow pen to be completely transparent or black. (0: transparent, 1: black)
 * @default 1
 * @type number
 *
 * @help This plugin does not provide plugin commands.
 * You can see the changes in the play screen, but not in the editor.
 * 
 * [License]
 * this plugin is released under MIT license.
 * http://opensource.org/licenses/mit-license.php
 */

/*:ja
 * @target MZ
 * @plugindesc 影ペンを完全に透明にするか、黒にすることができます。
 * @author Koji Marumugi
 * 
 * @param choice
 * @text 透明か黒か
 * @desc 影ペンを完全に透明にするか、黒にするか選びます。（0：透明、1：黒）
 * @default 1
 * @type number
 *
 * @help このプラグインには、プラグインコマンドはありません。
 * また、エディタ上では変化せず、プレイ画面にて変化を確認する事が出来ます。
 * 
 * ■ライセンス表記
 * このプラグインは MIT ライセンスで配布されます。
 * ご自由にお使いください。
 * http://opensource.org/licenses/mit-license.php
 */

(()=>{
    'use strict';

    const name = 'FC_ShadowPenMZ';
    const parameters = PluginManager.parameters(name);
    let choice = Number(parameters['choice'] || 0);

    if(choice === 0) {
        const _Tilemap_Renderer_createShader = Tilemap.Renderer.prototype._createShader;
        Tilemap.Renderer.prototype._createShader = function() {
       _Tilemap_Renderer_createShader.apply(this, arguments);
        const vertexSrc =
            "attribute float aTextureId;" +
            "attribute vec4 aFrame;" +
            "attribute vec2 aSource;" +
            "attribute vec2 aDest;" +
            "uniform mat3 uProjectionMatrix;" +
            "varying vec4 vFrame;" +
            "varying vec2 vTextureCoord;" +
            "varying float vTextureId;" +
            "void main(void) {" +
            "  vec3 position = uProjectionMatrix * vec3(aDest, 1.0);" +
            "  gl_Position = vec4(position, 1.0);" +
            "  vFrame = aFrame;" +
            "  vTextureCoord = aSource;" +
            "  vTextureId = aTextureId;" +
            "}";
        const fragmentSrc =
            "varying vec4 vFrame;" +
            "varying vec2 vTextureCoord;" +
            "varying float vTextureId;" +
            "uniform sampler2D uSampler0;" +
            "uniform sampler2D uSampler1;" +
            "uniform sampler2D uSampler2;" +
            "void main(void) {" +
            "  vec2 textureCoord = clamp(vTextureCoord, vFrame.xy, vFrame.zw);" +
            "  int textureId = int(vTextureId);" +
            "  vec4 color;" +
            "  if (textureId < 0) {" +
            "    color = vec4(0.0, 0.0, 0.0, 0);" +
            "  } else if (textureId == 0) {" +
            "    color = texture2D(uSampler0, textureCoord / 2048.0);" +
            "  } else if (textureId == 1) {" +
            "    color = texture2D(uSampler1, textureCoord / 2048.0);" +
            "  } else if (textureId == 2) {" +
            "    color = texture2D(uSampler2, textureCoord / 2048.0);" +
            "  }" +
            "  gl_FragColor = color;" +
            "}"
        
        return new PIXI.Shader(PIXI.Program.from(vertexSrc, fragmentSrc), {
            uSampler0: 0,
            uSampler1: 0,
            uSampler2: 0,
            uProjectionMatrix: new PIXI.Matrix() 
        });
    }; 

    }else if(choice === 1) {
        const _Tilemap_Renderer_createShader = Tilemap.Renderer.prototype._createShader;
        Tilemap.Renderer.prototype._createShader = function() {
       _Tilemap_Renderer_createShader.apply(this, arguments);
        const vertexSrc =
            "attribute float aTextureId;" +
            "attribute vec4 aFrame;" +
            "attribute vec2 aSource;" +
            "attribute vec2 aDest;" +
            "uniform mat3 uProjectionMatrix;" +
            "varying vec4 vFrame;" +
            "varying vec2 vTextureCoord;" +
            "varying float vTextureId;" +
            "void main(void) {" +
            "  vec3 position = uProjectionMatrix * vec3(aDest, 1.0);" +
            "  gl_Position = vec4(position, 1.0);" +
            "  vFrame = aFrame;" +
            "  vTextureCoord = aSource;" +
            "  vTextureId = aTextureId;" +
            "}";
        const fragmentSrc =
            "varying vec4 vFrame;" +
            "varying vec2 vTextureCoord;" +
            "varying float vTextureId;" +
            "uniform sampler2D uSampler0;" +
            "uniform sampler2D uSampler1;" +
            "uniform sampler2D uSampler2;" +
            "void main(void) {" +
            "  vec2 textureCoord = clamp(vTextureCoord, vFrame.xy, vFrame.zw);" +
            "  int textureId = int(vTextureId);" +
            "  vec4 color;" +
            "  if (textureId < 0) {" +
            "    color = vec4(0.0, 0.0, 0.0, 1);" +
            "  } else if (textureId == 0) {" +
            "    color = texture2D(uSampler0, textureCoord / 2048.0);" +
            "  } else if (textureId == 1) {" +
            "    color = texture2D(uSampler1, textureCoord / 2048.0);" +
            "  } else if (textureId == 2) {" +
            "    color = texture2D(uSampler2, textureCoord / 2048.0);" +
            "  }" +
            "  gl_FragColor = color;" +
            "}"
        
        return new PIXI.Shader(PIXI.Program.from(vertexSrc, fragmentSrc), {
            uSampler0: 0,
            uSampler1: 0,
            uSampler2: 0,
            uProjectionMatrix: new PIXI.Matrix() 
        });
        

    }; 

    };

    // const _Tilemap_Renderer_createShader = Tilemap.Renderer.prototype._createShader;
    // Tilemap.Renderer.prototype._createShader = function() {
    //    _Tilemap_Renderer_createShader.apply(this, arguments);
    //     const vertexSrc =
    //         "attribute float aTextureId;" +
    //         "attribute vec4 aFrame;" +
    //         "attribute vec2 aSource;" +
    //         "attribute vec2 aDest;" +
    //         "uniform mat3 uProjectionMatrix;" +
    //         "varying vec4 vFrame;" +
    //         "varying vec2 vTextureCoord;" +
    //         "varying float vTextureId;" +
    //         "void main(void) {" +
    //         "  vec3 position = uProjectionMatrix * vec3(aDest, 1.0);" +
    //         "  gl_Position = vec4(position, 1.0);" +
    //         "  vFrame = aFrame;" +
    //         "  vTextureCoord = aSource;" +
    //         "  vTextureId = aTextureId;" +
    //         "}";
    //     const fragmentSrc =
    //         "varying vec4 vFrame;" +
    //         "varying vec2 vTextureCoord;" +
    //         "varying float vTextureId;" +
    //         "uniform sampler2D uSampler0;" +
    //         "uniform sampler2D uSampler1;" +
    //         "uniform sampler2D uSampler2;" +
    //         "void main(void) {" +
    //         "  vec2 textureCoord = clamp(vTextureCoord, vFrame.xy, vFrame.zw);" +
    //         "  int textureId = int(vTextureId);" +
    //         "  vec4 color;" +
    //         "  if (textureId < 0) {" +
    //         "    color = vec4(0.0, 0.0, 0.0, 1);" +
    //         "  } else if (textureId == 0) {" +
    //         "    color = texture2D(uSampler0, textureCoord / 2048.0);" +
    //         "  } else if (textureId == 1) {" +
    //         "    color = texture2D(uSampler1, textureCoord / 2048.0);" +
    //         "  } else if (textureId == 2) {" +
    //         "    color = texture2D(uSampler2, textureCoord / 2048.0);" +
    //         "  }" +
    //         "  gl_FragColor = color;" +
    //         "}"
        
    //     return new PIXI.Shader(PIXI.Program.from(vertexSrc, fragmentSrc), {
    //         uSampler0: 0,
    //         uSampler1: 0,
    //         uSampler2: 0,
    //         uProjectionMatrix: new PIXI.Matrix() 
    //     });
        

    // }; 

})();